@@ -1,6 +1,6 @@
 # interpretation function for ergm objects
 interpret.ergm <- function(object, formula = object$formula, 
-    coefficients = coef(object), network = eval(parse(text = 
+    coefficients = coef(object), target = eval(parse(text = 
     deparse(formula[[2]]))), type = "tie", i, j, ...) {
   
   if (length(j) > 1 && (type == "tie" || type == "dyad")) {
@@ -9,9 +9,7 @@ interpret.ergm <- function(object, formula = object$formula,
   }
   
   # extract response network and adjust formula
-  #nw <- eval(parse(text = deparse(formula[[2]])))  # response network
-  nw <- network
-  #nw <- ergm.getnetwork(formula)
+  nw <- target
   dir <- is.directed(nw)
   if (dir == FALSE && type == "dyad") {
     type <- "tie"
@@ -130,55 +128,40 @@ interpret.ergm <- function(object, formula = object$formula,
 
 
 # interpretation method for btergm objects
-interpret.btergm <- function(object, formula = object@formula, 
-    coefficients = coef(object), network = eval(parse(text = 
-    deparse(formula[[2]]))), type = "tie", i, j, t = 1:length(network), 
-    ...) {
+interpret.btergm <- function(object, formula = getformula(object), 
+    coefficients = coef(object), target = NULL, type = "tie", i, j, 
+    t = 1:object@time.steps, ...) {
+  
+  env <- tergmprepare(formula = formula, offset = FALSE, blockdiag = FALSE, 
+      verbose = FALSE)
+  parent.env(env) <- environment()
   
   # extract response networks and adjust formula
-  networks <- network
-  if (class(networks) == "network" || class(networks) == "matrix") {
-    networks <- list(networks)
+  if (!is.null(target)) {
+    env$networks <- target
   }
-  dir <- is.directed(networks[[1]])
-  if (dir == FALSE && type == "dyad") {
-    type <- "tie"
-    warning(paste("Dyadic probabilities not available for undirected",
-        "networks. Reporting tie probabilities instead."))
-  }
-  
-  # disassemble formula and preprocess rhs
-  tilde <- deparse(formula[[1]])
-  lhs <- "networks[[k]]"
-  rhs <- paste(deparse(formula[[3]]), collapse = "")
-  rhs <- gsub("\\s+", " ", rhs)
-  rhs <- preprocessrhs(rhs, length(networks), iterator = "k")
-  
-  # reassemble formula
-  f <- paste(lhs, tilde, rhs)
-  form <- as.formula(f)
   
   # prepare i and j
   if (!is.list(i)) {
-    i <- rep(list(i), length(networks))
+    i <- rep(list(i), length(env$networks))
     num.actors <- numeric()
     for (k in t) {
-      num.actors[k] <- nrow(as.matrix(networks[[k]]))
+      num.actors[k] <- nrow(as.matrix(env$networks[[k]]))
     }
     if (length(table(num.actors)) > 1) {
       warning(paste("'i' does not vary across time steps, but the number of",
-          "actors does. 'i' can be provided as a list."))
+          "actors does. 'i' can be provided as a list or as a name."))
     }
   }
   if (!is.list(j)) {
-    j <- rep(list(j), length(networks))
+    j <- rep(list(j), length(env$networks))
     num.actors <- numeric()
     for (k in t) {
-      num.actors[k] <- nrow(as.matrix(networks[[k]]))
+      num.actors[k] <- nrow(as.matrix(env$networks[[k]]))
     }
     if (length(table(num.actors)) > 1) {
       warning(paste("'j' does not vary across time steps, but the number of",
-          "actors does. 'j' can be provided as a list."))
+          "actors does. 'j' can be provided as a list or as a name."))
     }
   }
   for (l in 1:length(j)) {
@@ -187,15 +170,16 @@ interpret.btergm <- function(object, formula = object@formula,
           "node can be specified per time step."))
     }
   }
+  node_i <- i
+  node_j <- j
   
-  # do the computations
   if (type == "tie") {
     results <- numeric()
-    for (k in t) {
-      networks[[k]][i[[k]], j[[k]]] <- 0
-      stat0 <- summary(remove.offset.formula(form), response = NULL)
-      networks[[k]][i[[k]], j[[k]]] <- 1
-      stat1 <- summary(remove.offset.formula(form), response = NULL)
+    for (i in t) {
+      env$networks[[i]][node_i[[i]], node_j[[i]]] <- 0
+      stat0 <- summary(remove.offset.formula(env$form), response = NULL)
+      env$networks[[i]][node_i[[i]], node_j[[i]]] <- 1
+      stat1 <- summary(remove.offset.formula(env$form), response = NULL)
       chgstat <- stat1 - stat0
       if (length(chgstat) != length(coefficients)) {
         stop(paste("Number of coefficients and statistics differ.",
@@ -205,20 +189,19 @@ interpret.btergm <- function(object, formula = object@formula,
       lp <- t(chgstat) %*% cbind(coefficients)
       result <- c(1 / (1 + exp(-lp)))
       names(result) <- "i->j = 1"
-      results[k] <- result
-      #names(results)[k] <- paste0("t = ", k)
+      results[i] <- result
     }
     results <- results[!is.na(results)]
     names(results) <- paste("t =", t)
   } else if (type == "dyad") {
     results <- list()
-    for (k in t) {
+    for (i in t) {
       eta_mat <- matrix(NA, 2, 2)
       for (xi in 0:1) {
         for (xj in 0:1) {
-          networks[[k]][i[[k]], j[[k]]] <- xi
-          networks[[k]][j[[k]], i[[k]]] <- xj
-          stat <- summary(remove.offset.formula(form), response = NULL)
+          env$networks[[i]][node_i[[i]], node_j[[i]]] <- xi
+          env$networks[[i]][node_j[[i]], node_i[[i]]] <- xj
+          stat <- summary(remove.offset.formula(env$form), response = NULL)
           if (length(stat) != length(coefficients)) {
             stop(paste("Number of coefficients and statistics differ.",
                 "Did you fit a curved model? Curved models with non-fixed",
@@ -236,24 +219,24 @@ interpret.btergm <- function(object, formula = object@formula,
       }
       rownames(prob_mat) <- c("i->j = 0", "i->j = 1")
       colnames(prob_mat) <- c("j->i = 0", "j->i = 1")
-      results[[k]] <- prob_mat
+      results[[i]] <- prob_mat
     }
     results <- results[!sapply(results, is.null)]
     names(results) <- paste("t =", t)
   } else if (type == "node") {
     results <- list()
-    for (k in t) {
-      m <- length(i[[k]])
-      n <- length(j[[k]])
+    for (i in t) {
+      m <- length(node_i[[i]])
+      n <- length(node_j[[i]])
       if (m == 1 && n > 1) {
         labels <- c("Sender", "Receiver")
       } else if (m > 1 && n == 1) {
         labels <- c("Receiver", "Sender")
-        j.old <- j[[k]]
-        j[[k]] <- i[[k]]
-        i[[k]] <- j.old
-        m <- length(i[[k]])
-        n <- length(j[[k]])
+        j.old <- node_j[[i]]
+        node_j[[i]] <- node_i[[i]]
+        node_i[[i]] <- j.old
+        m <- length(node_i[[i]])
+        n <- length(node_j[[i]])
       } else {
         stop(paste("Either 'i' or 'j' must contain more than one node per",
             "time step."))
@@ -270,10 +253,10 @@ interpret.btergm <- function(object, formula = object@formula,
       }
       eta <- numeric(nrow(vecs))
       for (l in 1:nrow(vecs)) {
-        ik <- i[[k]]
-        jk <- j[[k]]
-        networks[[k]][ik, jk] <- vecs[l, ]
-        stat <- summary(remove.offset.formula(form), response = NULL)
+        ik <- node_i[[i]]
+        jk <- node_j[[i]]
+        env$networks[[i]][ik, jk] <- vecs[l, ]
+        stat <- summary(remove.offset.formula(env$form), response = NULL)
         if (length(stat) != length(coefficients)) {
           stop(paste("Number of coefficients and statistics differ.",
               "Did you fit a curved model? Curved models with non-fixed",
@@ -285,11 +268,11 @@ interpret.btergm <- function(object, formula = object@formula,
       for (l in 1:nrow(vecs)) {
         prob[l] <- 1 / sum(exp(eta - eta[l]))
       }
-      colnames(vecs) <- paste(labels[2], j[[k]])
-      rownames(vecs) <- rep(paste(labels[1], i[[k]]), nrow(vecs))
+      colnames(vecs) <- paste(labels[2], node_j[[i]])
+      rownames(vecs) <- rep(paste(labels[1], node_i[[i]]), nrow(vecs))
       result <- cbind(prob, vecs)
       colnames(result)[1] <- "probability"
-      results[[k]] <- result
+      results[[i]] <- result
     }
     results <- results[!sapply(results, is.null)]
     names(results) <- paste("t =", t)
@@ -309,97 +292,3 @@ setMethod("interpret", signature = className("btergm", "btergm"),
 
 setMethod("interpret", signature = className("mtergm", "btergm"), 
     definition = interpret.btergm)
-
-
-# function which preprocesses the right-hand side (rhs) of the formula
-# (may be obsolete in the near future when the tergmprepare function is used)
-preprocessrhs <- function(rhs, time.steps, iterator = "i", dep = NULL) {
-  
-  covnames <- character()
-  
-  # split up rhs terms
-  rhs.terms <- strsplit(rhs, "\\s*(\\+|\\*)\\s*")[[1]]
-  rhs.indices <- gregexpr("\\+|\\*", rhs)[[1]]
-  if (length(rhs.indices) == 1 && rhs.indices < 0) {
-    rhs.operators <- character()
-  } else {
-    rhs.operators <- substring(rhs, rhs.indices, rhs.indices)
-  }
-  
-  # preprocess dyadcov and edgecov terms
-  for (k in 1:length(rhs.terms)) {
-    if (grepl("((edge)|(dyad))cov", rhs.terms[k])) {
-      if (grepl(",\\s*?((attr)|\\\")", rhs.terms[k])) { # with attrib argument
-        x1 <- sub("((?:offset\\()?((edge)|(dyad))cov\\()([^\\)]+)((,\\s*a*.*?)\\)(?:\\))?)", "\\1", 
-            rhs.terms[k], perl = TRUE)
-        x2 <- sub("((?:offset\\()?((edge)|(dyad))cov\\()([^\\)]+)((,\\s*a*.*?)\\)(?:\\))?)", "\\5", 
-            rhs.terms[k], perl = TRUE)
-        x3 <- sub("((?:offset\\()?((edge)|(dyad))cov\\()([^\\)]+)((,\\s*a*.*?)\\)(?:\\))?)", "\\6", 
-            rhs.terms[k], perl = TRUE)
-      } else { # without attribute argument
-        x1 <- sub("((?:offset\\()?((edge)|(dyad))cov\\()([^\\)]+)((,*\\s*a*.*?)\\)(?:\\))?)", "\\1", 
-            rhs.terms[k], perl = TRUE)
-        x2 <- sub("((?:offset\\()?((edge)|(dyad))cov\\()([^\\)]+)((,*\\s*a*.*?)\\)(?:\\))?)", "\\5", 
-            rhs.terms[k], perl = TRUE)
-        x3 <- sub("((?:offset\\()?((edge)|(dyad))cov\\()([^\\)]+)((,*\\s*a*.*?)\\)(?:\\))?)", "\\6", 
-            rhs.terms[k], perl = TRUE)
-      }
-      type <- class(eval(parse(text = x2)))
-      covnames <- c(covnames, x2)
-      
-      if (grepl("[^\\]]\\]$", x2)) {
-        # time-varying covariate with given indices (e.g., formula[1:5])
-        rhs.terms[k] <- paste(x1, x2, x3, sep = "")
-        if (length(eval(parse(text = x2))) != time.steps) {
-          stop(paste(x2, "has", length(eval(parse(text = x2))), 
-              "elements, but there are", time.steps, "networks to be modeled."))
-        }
-        
-      } else if (type == "matrix" || type == "network") {
-        # time-independent covariate
-        rhs.terms[k] <- paste(x1, x2, x3, sep = "")
-      } else if (type == "list" || type == "network.list") {
-        # time-varying covariate
-        if (length(eval(parse(text = x2))) != time.steps) {
-          stop(paste(x2, "has", length(get(x2)), "elements, but there are", 
-              time.steps, "networks to be modeled."))
-        }
-        x2 <- paste(x2, "[[", iterator, "]]", sep = "")
-        rhs.terms[k] <- paste(x1, x2, x3, sep = "")
-      } else {
-        stop(paste(x2, "is not a matrix, network, or list."))
-      }
-      
-      # check if dimensions at each time step are OK
-      if (!is.null(dep)) {
-        for (i in 1:length(dep)) {
-          cv <- eval(parse(text = x2))
-          msg <- paste0("btergm error: The dimensions of covariate '", x2, 
-              "' do not match the dimensions of the dependent network ", 
-              "at time step ", i, ".")
-          if ("list" %in% class(cv)) {
-            if (any(dim(as.matrix(dep[[i]])) != dim(as.matrix(cv[[i]])))) {
-              stop(msg)
-            }
-          } else {
-            if (any(dim(as.matrix(dep[[i]])) != dim(as.matrix(cv)))) {
-              stop(msg)
-            }
-          }
-        }
-      }
-    }
-  }
-  
-  assign("covnames", covnames, envir = parent.frame(n = 1))
-  
-  # reassemble rhs
-  rhs <- rhs.terms[1]
-  if (length(rhs.operators) > 0) {
-    for (i in 1:length(rhs.operators)) {
-      rhs <- paste(rhs, rhs.operators[i], rhs.terms[i + 1])
-    }
-  }
-  return(rhs)
-}
-
